/**
 * vis-data
 * http://visjs.org/
 *
 * Manage unstructured data using DataSet. Add, update, and remove data, and listen for changes in the data.
 *
 * @version 7.0.1
 * @date    2020-10-13T17:07:07.867Z
 *
 * @copyright (c) 2011-2017 Almende B.V, http://almende.com
 * @copyright (c) 2017-2019 visjs contributors, https://github.com/visjs
 *
 * @license
 * vis.js is dual licensed under both
 *
 *   1. The Apache 2.0 License
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *   and
 *
 *   2. The MIT License
 *      http://opensource.org/licenses/MIT
 *
 * vis.js may be distributed under either license.
 */
!function(t,e){"object"==typeof exports&&"undefined"!=typeof module?e(exports,require("uuid"),require("vis-util/esnext/umd/vis-util.js")):"function"==typeof define&&define.amd?define(["exports","uuid","vis-util/esnext/umd/vis-util.js"],e):e((t="undefined"!=typeof globalThis?globalThis:t||self).vis=t.vis||{},t.uuidv4,t.vis)}(this,(function(t,e,s){
/*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
function i(t,e){if(!e.has(t))throw new TypeError("attempted to get private field on non-instance");return e.get(t)}function r(t,e,s){if(!e.has(t))throw new TypeError("attempted to set private field on non-instance");return e.set(t,s),s}var n,o,h,l,a,u,d,f,c,p,g,m,y,_;class w{constructor(t,e,s){this._source=t,this._transformers=e,this._target=s,this._listeners={add:this._add.bind(this),remove:this._remove.bind(this),update:this._update.bind(this)}}all(){return this._target.update(this._transformItems(this._source.get())),this}start(){return this._source.on("add",this._listeners.add),this._source.on("remove",this._listeners.remove),this._source.on("update",this._listeners.update),this}stop(){return this._source.off("add",this._listeners.add),this._source.off("remove",this._listeners.remove),this._source.off("update",this._listeners.update),this}_transformItems(t){return this._transformers.reduce(((t,e)=>e(t)),t)}_add(t,e){null!=e&&this._target.add(this._transformItems(this._source.get(e.items)))}_update(t,e){null!=e&&this._target.update(this._transformItems(this._source.get(e.items)))}_remove(t,e){null!=e&&this._target.remove(this._transformItems(e.oldData))}}class v{constructor(t){this._source=t,n.set(this,[])}filter(t){return i(this,n).push((e=>e.filter(t))),this}map(t){return i(this,n).push((e=>e.map(t))),this}flatMap(t){return i(this,n).push((e=>e.flatMap(t))),this}to(t){return new w(this._source,i(this,n),t)}}function b(t){return"string"==typeof t||"number"==typeof t}n=new WeakMap;class D{constructor(t){o.set(this,[]),h.set(this,null),l.set(this,null),this.delay=null,this.max=1/0,this.setOptions(t)}setOptions(t){t&&void 0!==t.delay&&(this.delay=t.delay),t&&void 0!==t.max&&(this.max=t.max),this._flushIfNeeded()}static extend(t,e){const s=new D(e);if(void 0!==t.flush)throw new Error("Target object already has a property flush");t.flush=()=>{s.flush()};const i=[{name:"flush",original:void 0}];if(e&&e.replace)for(let r=0;r<e.replace.length;r++){const n=e.replace[r];i.push({name:n,original:t[n]}),s.replace(t,n)}return r(s,l,{object:t,methods:i}),s}destroy(){if(this.flush(),i(this,l)){const t=i(this,l).object,e=i(this,l).methods;for(let s=0;s<e.length;s++){const i=e[s];i.original?t[i.name]=i.original:delete t[i.name]}r(this,l,null)}}replace(t,e){const s=this,i=t[e];if(!i)throw new Error("Method "+e+" undefined");t[e]=function(...t){s.queue({args:t,fn:i,context:this})}}queue(t){"function"==typeof t?i(this,o).push({fn:t}):i(this,o).push(t),this._flushIfNeeded()}_flushIfNeeded(){i(this,o).length>this.max&&this.flush(),null!=i(this,h)&&(clearTimeout(i(this,h)),r(this,h,null)),this.queue.length>0&&"number"==typeof this.delay&&r(this,h,setTimeout((()=>{this.flush()}),this.delay))}flush(){i(this,o).splice(0).forEach((t=>{t.fn.apply(t.context||t.fn,t.args||[])}))}}o=new WeakMap,h=new WeakMap,l=new WeakMap;class E{constructor(){a.set(this,{"*":[],add:[],remove:[],update:[]}),this.subscribe=E.prototype.on,this.unsubscribe=E.prototype.off}_trigger(t,e,s){if("*"===t)throw new Error("Cannot trigger event *");[...i(this,a)[t],...i(this,a)["*"]].forEach((i=>{i(t,e,null!=s?s:null)}))}on(t,e){"function"==typeof e&&i(this,a)[t].push(e)}off(t,e){i(this,a)[t]=i(this,a)[t].filter((t=>t!==e))}}a=new WeakMap;class I{constructor(t){u.set(this,void 0),r(this,u,t)}*[(u=new WeakMap,Symbol.iterator)](){for(const[t,e]of i(this,u))yield[t,e]}*entries(){for(const[t,e]of i(this,u))yield[t,e]}*keys(){for(const[t]of i(this,u))yield t}*values(){for(const[,t]of i(this,u))yield t}toIdArray(){return[...i(this,u)].map((t=>t[0]))}toItemArray(){return[...i(this,u)].map((t=>t[1]))}toEntryArray(){return[...i(this,u)]}toObjectMap(){const t=Object.create(null);for(const[e,s]of i(this,u))t[e]=s;return t}toMap(){return new Map(i(this,u))}toIdSet(){return new Set(this.toIdArray())}toItemSet(){return new Set(this.toItemArray())}cache(){return new I([...i(this,u)])}distinct(t){const e=new Set;for(const[s,r]of i(this,u))e.add(t(r,s));return e}filter(t){const e=i(this,u);return new I({*[Symbol.iterator](){for(const[s,i]of e)t(i,s)&&(yield[s,i])}})}forEach(t){for(const[e,s]of i(this,u))t(s,e)}map(t){const e=i(this,u);return new I({*[Symbol.iterator](){for(const[s,i]of e)yield[s,t(i,s)]}})}max(t){const e=i(this,u)[Symbol.iterator]();let s=e.next();if(s.done)return null;let r=s.value[1],n=t(s.value[1],s.value[0]);for(;!(s=e.next()).done;){const[e,i]=s.value,o=t(i,e);o>n&&(n=o,r=i)}return r}min(t){const e=i(this,u)[Symbol.iterator]();let s=e.next();if(s.done)return null;let r=s.value[1],n=t(s.value[1],s.value[0]);for(;!(s=e.next()).done;){const[e,i]=s.value,o=t(i,e);o<n&&(n=o,r=i)}return r}reduce(t,e){for(const[s,r]of i(this,u))e=t(e,r,s);return e}sort(t){return new I({[Symbol.iterator]:()=>[...i(this,u)].sort((([e,s],[i,r])=>t(s,r,e,i)))[Symbol.iterator]()})}}class k extends E{constructor(t,e){super(),d.set(this,void 0),f.set(this,void 0),c.set(this,void 0),p.set(this,null),t&&!Array.isArray(t)&&(e=t,t=[]),r(this,d,e||{}),r(this,f,new Map),this.length=0,r(this,c,i(this,d).fieldId||"id"),t&&t.length&&this.add(t),this.setOptions(e)}setOptions(t){t&&void 0!==t.queue&&(!1===t.queue?i(this,p)&&(i(this,p).destroy(),r(this,p,null)):(i(this,p)||r(this,p,D.extend(this,{replace:["add","update","remove"]})),t.queue&&"object"==typeof t.queue&&i(this,p).setOptions(t.queue)))}add(t,e){const s=[];let r;if(Array.isArray(t)){if(t.map((t=>t[i(this,c)])).some((t=>i(this,f).has(t))))throw new Error("A duplicate id was found in the parameter array.");for(let e=0,i=t.length;e<i;e++)r=this._addItem(t[e]),s.push(r)}else{if(!t||"object"!=typeof t)throw new Error("Unknown dataType");r=this._addItem(t),s.push(r)}return s.length&&this._trigger("add",{items:s},e),s}update(t,e){const s=[],r=[],n=[],o=[],h=i(this,c),l=t=>{const e=t[h];if(null!=e&&i(this,f).has(e)){const s=t,h=Object.assign({},i(this,f).get(e)),l=this._updateItem(s);r.push(l),o.push(s),n.push(h)}else{const e=this._addItem(t);s.push(e)}};if(Array.isArray(t))for(let e=0,s=t.length;e<s;e++)t[e]&&"object"==typeof t[e]?l(t[e]):console.warn("Ignoring input item, which is not an object at index "+e);else{if(!t||"object"!=typeof t)throw new Error("Unknown dataType");l(t)}if(s.length&&this._trigger("add",{items:s},e),r.length){const t={items:r,oldData:n,data:o};this._trigger("update",t,e)}return s.concat(r)}updateOnly(t,e){Array.isArray(t)||(t=[t]);const r=t.map((t=>{const e=i(this,f).get(t[i(this,c)]);if(null==e)throw new Error("Updating non-existent items is not allowed.");return{oldData:e,update:t}})).map((({oldData:t,update:e})=>{const r=t[i(this,c)],n=s.pureDeepObjectAssign(t,e);return i(this,f).set(r,n),{id:r,oldData:t,updatedData:n}}));if(r.length){const t={items:r.map((t=>t.id)),oldData:r.map((t=>t.oldData)),data:r.map((t=>t.updatedData))};return this._trigger("update",t,e),t.items}return[]}get(t,e){let s=void 0,r=void 0,n=void 0;b(t)?(s=t,n=e):Array.isArray(t)?(r=t,n=e):n=t;const o=n&&"Object"===n.returnType?"Object":"Array",h=n&&n.filter,l=[];let a=void 0,u=void 0,d=void 0;if(null!=s)a=i(this,f).get(s),a&&h&&!h(a)&&(a=void 0);else if(null!=r)for(let t=0,e=r.length;t<e;t++)a=i(this,f).get(r[t]),null==a||h&&!h(a)||l.push(a);else{u=[...i(this,f).keys()];for(let t=0,e=u.length;t<e;t++)d=u[t],a=i(this,f).get(d),null==a||h&&!h(a)||l.push(a)}if(n&&n.order&&null==s&&this._sort(l,n.order),n&&n.fields){const t=n.fields;if(null!=s&&null!=a)a=this._filterFields(a,t);else for(let e=0,s=l.length;e<s;e++)l[e]=this._filterFields(l[e],t)}if("Object"==o){const t={};for(let e=0,s=l.length;e<s;e++){const s=l[e];t[s[i(this,c)]]=s}return t}return null!=s?null!=a?a:null:l}getIds(t){const e=i(this,f),s=t&&t.filter,r=t&&t.order,n=[...e.keys()],o=[];if(s)if(r){const t=[];for(let e=0,r=n.length;e<r;e++){const r=n[e],o=i(this,f).get(r);null!=o&&s(o)&&t.push(o)}this._sort(t,r);for(let e=0,s=t.length;e<s;e++)o.push(t[e][i(this,c)])}else for(let t=0,e=n.length;t<e;t++){const e=n[t],r=i(this,f).get(e);null!=r&&s(r)&&o.push(r[i(this,c)])}else if(r){const t=[];for(let s=0,i=n.length;s<i;s++){const i=n[s];t.push(e.get(i))}this._sort(t,r);for(let e=0,s=t.length;e<s;e++)o.push(t[e][i(this,c)])}else for(let t=0,s=n.length;t<s;t++){const s=n[t],r=e.get(s);null!=r&&o.push(r[i(this,c)])}return o}getDataSet(){return this}forEach(t,e){const s=e&&e.filter,r=[...i(this,f).keys()];if(e&&e.order){const s=this.get(e);for(let e=0,r=s.length;e<r;e++){const r=s[e];t(r,r[i(this,c)])}}else for(let e=0,n=r.length;e<n;e++){const n=r[e],o=i(this,f).get(n);null==o||s&&!s(o)||t(o,n)}}map(t,e){const s=e&&e.filter,r=[],n=[...i(this,f).keys()];for(let e=0,o=n.length;e<o;e++){const o=n[e],h=i(this,f).get(o);null==h||s&&!s(h)||r.push(t(h,o))}return e&&e.order&&this._sort(r,e.order),r}_filterFields(t,e){return t?(Array.isArray(e)?e:Object.keys(e)).reduce(((e,s)=>(e[s]=t[s],e)),{}):t}_sort(t,e){if("string"==typeof e){const s=e;t.sort(((t,e)=>{const i=t[s],r=e[s];return i>r?1:i<r?-1:0}))}else{if("function"!=typeof e)throw new TypeError("Order must be a function or a string");t.sort(e)}}remove(t,e){const s=[],r=[],n=Array.isArray(t)?t:[t];for(let t=0,e=n.length;t<e;t++){const e=this._remove(n[t]);if(e){const t=e[i(this,c)];null!=t&&(s.push(t),r.push(e))}}return s.length&&this._trigger("remove",{items:s,oldData:r},e),s}_remove(t){let e;if(b(t)?e=t:t&&"object"==typeof t&&(e=t[i(this,c)]),null!=e&&i(this,f).has(e)){const t=i(this,f).get(e)||null;return i(this,f).delete(e),--this.length,t}return null}clear(t){const e=[...i(this,f).keys()],s=[];for(let t=0,r=e.length;t<r;t++)s.push(i(this,f).get(e[t]));return i(this,f).clear(),this.length=0,this._trigger("remove",{items:e,oldData:s},t),e}max(t){let e=null,s=null;for(const r of i(this,f).values()){const i=r[t];"number"==typeof i&&(null==s||i>s)&&(e=r,s=i)}return e||null}min(t){let e=null,s=null;for(const r of i(this,f).values()){const i=r[t];"number"==typeof i&&(null==s||i<s)&&(e=r,s=i)}return e||null}distinct(t){const e=i(this,f),s=[...e.keys()],r=[];let n=0;for(let i=0,o=s.length;i<o;i++){const o=s[i],h=e.get(o)[t];let l=!1;for(let t=0;t<n;t++)if(r[t]==h){l=!0;break}l||void 0===h||(r[n]=h,n++)}return r}_addItem(t){const s=function(t,s){return null==t[s]&&(t[s]=e.v4()),t}(t,i(this,c)),r=s[i(this,c)];if(i(this,f).has(r))throw new Error("Cannot add item: item with id "+r+" already exists");return i(this,f).set(r,s),++this.length,r}_updateItem(t){const e=t[i(this,c)];if(null==e)throw new Error("Cannot update item: item has no id (item: "+JSON.stringify(t)+")");const s=i(this,f).get(e);if(!s)throw new Error("Cannot update item: no item with id "+e+" found");return i(this,f).set(e,{...s,...t}),e}stream(t){if(t){const e=i(this,f);return new I({*[Symbol.iterator](){for(const s of t){const t=e.get(s);null!=t&&(yield[s,t])}}})}return new I({[Symbol.iterator]:i(this,f).entries.bind(i(this,f))})}}d=new WeakMap,f=new WeakMap,c=new WeakMap,p=new WeakMap;class x extends E{constructor(t,e){super(),this.length=0,g.set(this,void 0),m.set(this,void 0),y.set(this,new Set),_.set(this,void 0),r(this,_,e||{}),r(this,g,this._onEvent.bind(this)),this.setData(t)}setData(t){if(i(this,m)){i(this,m).off&&i(this,m).off("*",i(this,g));const t=i(this,m).getIds({filter:i(this,_).filter}),e=i(this,m).get(t);i(this,y).clear(),this.length=0,this._trigger("remove",{items:t,oldData:e})}if(null!=t){r(this,m,t);const e=i(this,m).getIds({filter:i(this,_).filter});for(let t=0,s=e.length;t<s;t++){const s=e[t];i(this,y).add(s)}this.length=e.length,this._trigger("add",{items:e})}else r(this,m,new k);i(this,m).on&&i(this,m).on("*",i(this,g))}refresh(){const t=i(this,m).getIds({filter:i(this,_).filter}),e=[...i(this,y)],s={},r=[],n=[],o=[];for(let e=0,n=t.length;e<n;e++){const n=t[e];s[n]=!0,i(this,y).has(n)||(r.push(n),i(this,y).add(n))}for(let t=0,r=e.length;t<r;t++){const r=e[t],h=i(this,m).get(r);null==h?console.error("If you see this, report it please."):s[r]||(n.push(r),o.push(h),i(this,y).delete(r))}this.length+=r.length-n.length,r.length&&this._trigger("add",{items:r}),n.length&&this._trigger("remove",{items:n,oldData:o})}get(t,e){if(null==i(this,m))return null;let s,r=null;b(t)||Array.isArray(t)?(r=t,s=e):s=t;const n=Object.assign({},i(this,_),s),o=i(this,_).filter,h=s&&s.filter;return o&&h&&(n.filter=t=>o(t)&&h(t)),null==r?i(this,m).get(n):i(this,m).get(r,n)}getIds(t){if(i(this,m).length){const e=i(this,_).filter,s=null!=t?t.filter:null;let r;return r=s?e?t=>e(t)&&s(t):s:e,i(this,m).getIds({filter:r,order:t&&t.order})}return[]}forEach(t,e){if(i(this,m)){const s=i(this,_).filter,r=e&&e.filter;let n;n=r?s?function(t){return s(t)&&r(t)}:r:s,i(this,m).forEach(t,{filter:n,order:e&&e.order})}}map(t,e){if(i(this,m)){const s=i(this,_).filter,r=e&&e.filter;let n;return n=r?s?t=>s(t)&&r(t):r:s,i(this,m).map(t,{filter:n,order:e&&e.order})}return[]}getDataSet(){return i(this,m).getDataSet()}stream(t){return i(this,m).stream(t||{[Symbol.iterator]:i(this,y).keys.bind(i(this,y))})}dispose(){var t;(null===(t=i(this,m))||void 0===t?void 0:t.off)&&i(this,m).off("*",i(this,g));const e="This data view has already been disposed of.",s={get:()=>{throw new Error(e)},set:()=>{throw new Error(e)},configurable:!1};for(const t of Reflect.ownKeys(x.prototype))Object.defineProperty(this,t,s)}_onEvent(t,e,s){if(!e||!e.items||!i(this,m))return;const r=e.items,n=[],o=[],h=[],l=[],a=[],u=[];switch(t){case"add":for(let t=0,e=r.length;t<e;t++){const e=r[t];this.get(e)&&(i(this,y).add(e),n.push(e))}break;case"update":for(let t=0,s=r.length;t<s;t++){const s=r[t];this.get(s)?i(this,y).has(s)?(o.push(s),a.push(e.data[t]),l.push(e.oldData[t])):(i(this,y).add(s),n.push(s)):i(this,y).has(s)&&(i(this,y).delete(s),h.push(s),u.push(e.oldData[t]))}break;case"remove":for(let t=0,s=r.length;t<s;t++){const s=r[t];i(this,y).has(s)&&(i(this,y).delete(s),h.push(s),u.push(e.oldData[t]))}}this.length+=n.length-h.length,n.length&&this._trigger("add",{items:n},s),o.length&&this._trigger("update",{items:o,oldData:l,data:a},s),h.length&&this._trigger("remove",{items:h,oldData:u},s)}}g=new WeakMap,m=new WeakMap,y=new WeakMap,_=new WeakMap,Object.defineProperty(t,"DELETE",{enumerable:!0,get:function(){return s.DELETE}}),t.DataSet=k,t.DataStream=I,t.DataView=x,t.Queue=D,t.createNewDataPipeFrom=function(t){return new v(t)},Object.defineProperty(t,"__esModule",{value:!0})}));
//# sourceMappingURL=vis-data.min.js.map
